/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: gui_slider.c,v 1.9 2000/12/11 19:37:53 f1rmb Exp $
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include "gui_widget.h"
#include "gui_slider.h"
#include "gui_widget_types.h"
#include "gui_main.h"

extern Display *gDisplay;
extern uint32_t xine_debug;

/* ------------------------------------------------------------------------- */
/*
 * Draw widget
 */
void paint_slider (widget_t *b, Window win, GC gc) {
  float tmp = 0L;
  int button_width, button_height;	
  slider_private_data_t *private_data = 
    (slider_private_data_t *) b->private_data;
  gui_image_t *bg = (gui_image_t *) private_data->bg_skin;
  gui_image_t *paddle = (gui_image_t *) private_data->paddle_skin;

  XLockDisplay (gDisplay);
  
  button_width = private_data->button_width;
  button_height = private_data->paddle_skin->height;
  
  if(private_data->sType == HSLIDER)
    tmp = b->width - button_width;
  if(private_data->sType == VSLIDER)
    tmp = b->height - button_height;

  tmp /= private_data->max;
  tmp *= private_data->pos;

  XCopyArea (gDisplay, bg->image, win, gc, 0, 0,
	     bg->width, bg->height, b->x, b->y);
  
  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    int x=0, y=0;
    if(private_data->sType == HSLIDER) {
      x = b->x + (int)tmp;
      y = b->y;
    }
    else if(private_data->sType == VSLIDER) {
      x = b->x;
      y = b->y + private_data->bg_skin->height - button_height - (int)tmp;
    }
    if (private_data->bArmed) {
      if (private_data->bClicked) {
	XCopyArea (gDisplay, paddle->image, win, gc, 2*button_width, 0,
		   button_width, paddle->height, x, y);
	
      } else {
	XCopyArea (gDisplay, paddle->image, win, gc, button_width, 0,
		   button_width, paddle->height, x, y);
      }
    } else {
      XCopyArea (gDisplay, paddle->image, win, gc, 0, 0,
		 button_width, paddle->height, x, y);
    }
    XSetForeground(gDisplay, gc, gui_color.white.pixel);
    
    XFlush (gDisplay);
    
  } 
  else
    xprintf (VERBOSE|GUI, "paint slider on something (%d) "
	     "that is not a slider\n", b->widget_type);
  
  XUnlockDisplay (gDisplay);

}
/* ------------------------------------------------------------------------- */
/*
 * Got click
 */
int notify_click_slider (widget_t *b, int bUp, int x, int y) {
  slider_private_data_t *private_data = 
    (slider_private_data_t *) b->private_data;

  if (b->widget_type == WIDGET_TYPE_SLIDER) {

    if(private_data->sType == HSLIDER)
      private_data->pos = (int) ((x - b->x) * private_data->ratio);
    else if(private_data->sType == VSLIDER)
      private_data->pos = (int) ((b->y + private_data->bg_skin->height)
				 - (y - b->y - 
				    (private_data->paddle_skin->height/2)) 
				 * private_data->ratio);
    else
      xprintf(VERBOSE|GUI, "Unknown slider type (%d)\n", private_data->sType);
    
    if(private_data->pos > private_data->max) {
      xprintf(VERBOSE|GUI, "slider POS > MAX!.\n");
      private_data->pos = private_data->max;
    }
    if(private_data->pos < private_data->min) {
      xprintf(VERBOSE|GUI, "slider POS < MIN!.\n");
      private_data->pos = private_data->min;
    }

    private_data->bClicked = !bUp;
    
    if (bUp && private_data->bArmed && private_data->function)
      private_data->function (private_data->sWidget,
			      private_data->user_data,
			      private_data->pos);
  }
  else
    xprintf (VERBOSE|GUI, "notify click slider on something (%d) "
	     "that is not a slider\n", b->widget_type);
  return 1;
}
/* ------------------------------------------------------------------------- */
/*
 * Got focus
 */
int notify_focus_slider (widget_t *b, int bEntered) {
  slider_private_data_t *private_data = 
    (slider_private_data_t *) b->private_data;

  if (b->widget_type == WIDGET_TYPE_SLIDER)
    private_data->bArmed = bEntered;    
  else
    xprintf (VERBOSE|GUI, "notify slider button on something (%d) "
	     "that is not a slider\n", b->widget_type);
  return 1;
}
/* ------------------------------------------------------------------------- */
/*
 * Increment position
 */
void slider_make_step(widget_list_t *wl, widget_t *b) {
  slider_private_data_t *private_data = 
    (slider_private_data_t *) b->private_data;

  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    slider_set_pos(wl, b, slider_get_pos(b) + private_data->step);
  } 
  else
    xprintf (VERBOSE|GUI, "slider_make_step on something (%d) "
	     "that is not a slider\n", b->widget_type);
}
/* ------------------------------------------------------------------------- */
/*
 * Set value MIN.
 */
void slider_set_min(widget_t *b, int min) {
  slider_private_data_t *private_data =
    (slider_private_data_t *) b->private_data;
  
  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    if(min < private_data->max) {
      private_data->min = min;
      private_data->ratio  = 
	(double) (private_data->max-private_data->min) 
	/ private_data->bg_skin->width;
    }
  } else
    xprintf(VERBOSE|GUI, "slider_set_min on something (%d) that is not a slider\n",
	    b->widget_type);
}
/* ------------------------------------------------------------------------- */
/*
 * Set value MAX
 */
void slider_set_max(widget_t *b, int max) {
  slider_private_data_t *private_data =
    (slider_private_data_t *) b->private_data;
  
  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    if(max > private_data->min) {
      private_data->max = max;
      private_data->ratio  = 
	(double) (private_data->max-private_data->min) 
	/ private_data->bg_skin->width;
    }
  } 
  else
    xprintf(VERBOSE|GUI, "slider_set_min on something (%d) that is not a slider\n",
	    b->widget_type);
}
/* ------------------------------------------------------------------------- */
/*
 * Set pos to 0 and redraw the widget.
 */
void slider_reset(widget_list_t *wl, widget_t *b) {
  slider_private_data_t *private_data = 
    (slider_private_data_t *) b->private_data;
  
  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    private_data->pos = 0;
    paint_slider(b, wl->win, wl->gc);
  }
  else
    xprintf(VERBOSE|GUI, "slider_set_min on something (%d) that is not a slider\n",
	    b->widget_type);
}
/* ------------------------------------------------------------------------- */
/*
 * Return current position.
 */
int slider_get_pos(widget_t *b) {
  slider_private_data_t *private_data = 
    (slider_private_data_t *) b->private_data;

  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    return private_data->pos;
  } 
  else
    xprintf (VERBOSE|GUI, "slider_get_pos on something (%d) "
	     "that is not a slider\n", b->widget_type);
  return -1;
}
/* ------------------------------------------------------------------------- */
/*
 * Set position.
 */
void slider_set_pos(widget_list_t *wl, widget_t *b, int pos) {
  slider_private_data_t *private_data = 
    (slider_private_data_t *) b->private_data;

  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    if (pos >= 0 && pos <= private_data->max) {
      private_data->pos = pos;
      paint_slider(b, wl->win, wl->gc);
    }
    else
      slider_reset(wl, b);    
  } 
  else
    xprintf (VERBOSE|GUI, "slider_set_pos on something (%d) "
	     "that is not a slider\n", b->widget_type);
}
/* ------------------------------------------------------------------------- */
/*
 * Create the widget
 */
widget_t *create_slider (int type, int x, int y, 
			 int min, int max, int step, const char *bg, 
			 const char *paddle, void *f, void *ud) {

  widget_t                *mywidget;
  slider_private_data_t   *private_data;

  mywidget = (widget_t *) malloc (sizeof(widget_t));
  private_data = (slider_private_data_t *) 
    malloc (sizeof (slider_private_data_t));
  
  private_data->sWidget      = mywidget;
  private_data->sType        = type;
  private_data->bClicked     = 0;
  private_data->bArmed       = 0;
  private_data->min          = min;
  if(max <= min) 
    private_data->max        = min+1;
  else
    private_data->max        = max;
  private_data->pos          = 0;
  private_data->step         = step;
  private_data->paddle_skin  = gui_load_image(paddle);
  private_data->button_width = private_data->paddle_skin->width / 3;

  private_data->bg_skin      = gui_load_image(bg);

  if(type == HSLIDER)
    private_data->ratio      = (double)(max-min)/private_data->bg_skin->width;
  else if(type == VSLIDER)
    private_data->ratio      = (double)(max-min)/private_data->bg_skin->height;
  else
    xprintf(VERBOSE|GUI, "Unknown slider type (%d)\n", type);

  private_data->function     = f;
  private_data->user_data    = ud;

  mywidget->private_data     = private_data;

  mywidget->x                = x;
  mywidget->y                = y;


  mywidget->width            = private_data->bg_skin->width;
  mywidget->height           = private_data->bg_skin->height;
  mywidget->widget_type      = WIDGET_TYPE_SLIDER;
  mywidget->paint            = paint_slider;
  mywidget->notify_click     = notify_click_slider;
  mywidget->notify_focus     = notify_focus_slider;

  return mywidget;
}
/* ------------------------------------------------------------------------- */
