/*
 * yuv2rgb_mmx.c, Software YUV to RGB coverter with Intel MMX "technology"
 *
 * based on code from:
 * Copyright (C) 2000, Silicon Integrated System Corp.
 * All Rights Reserved.
 *
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: yuv2rgb_mmx.c,v 1.11 2000/12/08 22:59:13 guenter Exp $
 *
 *
 */

#include "config.h"

#ifdef __i386__

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "mmx.h"
#include "yuv2rgb.h"
#include "yuv2rgb_mmx.h"

#define movntq "movq" // for MMX-only processors
//#define movntq "movntq" // use this for processors that have SSE or 3Dnow

static void yuv420_rgb16_mmx_old  (uint8_t *image,
				   uint8_t *py, uint8_t *pu, uint8_t *pv,
				   int rgb_stride, int y_stride, int uv_stride)
{
  int even = 1;
  int y = 0;
  
  for (y=0; y<16; y++) {
    
      /*
       * this code deals with SINGLE scan line at a time,
       * it converts 8 pixels in each iteration
	     */
    __asm__ (".align 8\n\t"
	     MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    __asm__ (MMX_YUV_MUL : :);
    __asm__ (MMX_YUV_ADD : :);
    __asm__ (MMX_UNPACK_16RGB : : "r" (image));
    
    py += 8;
    pu += 4;
    pv += 4;
    image += 16;
    __asm__ (".align 8\n\t"
	     MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    __asm__ (MMX_YUV_MUL : :);
    __asm__ (MMX_YUV_ADD : :);
    __asm__ (MMX_UNPACK_16RGB : : "r" (image));
    
    py += 8;
    pu += 4;
    pv += 4;
    image += 16;
    
    if (even) {
      pu -= 8;
      pv -= 8;
    } else {
      pu += (uv_stride - 8);
      pv += (uv_stride - 8);
    }
    
    py += (y_stride - 16);
    image += (rgb_stride - 2*16);
    
    even = (!even);
  };
}

static void yuv420_rgb16_mmx (uint8_t *image,
			      uint8_t *py, uint8_t *pu, uint8_t *pv,
			      int rgb_stride, int y_stride, int uv_stride)
{
  int even = 1;
  int y = 0;
  
  for (y=0; y<16; y++) {
    
      /*
       * this code deals with SINGLE scan line at a time,
       * it converts 8 pixels in each iteration
	     */
    __asm__ (".align 8\n\t"
	     MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    __asm__ (MMX_YUV_MUL : :);
    __asm__ (MMX_YUV_ADD : :);
    __asm__ (MMX_UNPACK_16RGB : : "r" (image));
    
    py += 8;
    pu += 4;
    pv += 4;
    image += 16;
    __asm__ (".align 8\n\t"
	     MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    __asm__ (MMX_YUV_MUL : :);
    __asm__ (MMX_YUV_ADD : :);
    __asm__ (MMX_UNPACK_16RGB : : "r" (image));
    
    if (even) {
      pu -= 4;
      pv -= 4;
    } else {
      pu += (uv_stride - 4);
      pv += (uv_stride - 4);
    }
    
    py += (y_stride - 8);
    image += (rgb_stride - 16);
    
    even = (!even);
  };
}

static void yuv420_rgb16_mmx_anamorphic (uint8_t *image,
					 uint8_t *py, uint8_t *pu, 
					 uint8_t *pv,
					 int rgb_stride, int y_stride, 
					 int uv_stride)
{
  int even = 1;
  int y = 0;
  
  for (y=0; y<16; y++) {
    
      /*
       * this code deals with SINGLE scan line at a time,
       * it converts 8 pixels in each iteration
	     */
    __asm__ (".align 8\n\t"
	     MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    __asm__ (MMX_YUV_MUL : :);
    __asm__ (MMX_YUV_ADD : :);
    __asm__ (MMX_UNPACK_16RGB_ANAMORPHIC : : "r" (image));
    
    py += 8;
    pu += 4;
    pv += 4;
    image += 20;
    __asm__ (".align 8\n\t"
	     MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    __asm__ (MMX_YUV_MUL : :);
    __asm__ (MMX_YUV_ADD : :);
    __asm__ (MMX_UNPACK_16RGB_ANAMORPHIC : : "r" (image));
    
    if (even) {
      pu -= 4;
      pv -= 4;
    } else {
      pu += (uv_stride - 4);
      pv += (uv_stride - 4);
    }
    
    py += (y_stride - 8);
    image += (rgb_stride - 20);
    
    even = (!even);
  };
}

static void yuv420_bgr16_mmx (uint8_t *image,
			      uint8_t *py, uint8_t *pu, uint8_t *pv,
			      int rgb_stride, int y_stride, int uv_stride)
{
  int even = 1;
  int x = 0, y = 0;

  for (y=0; y<16; y++) {

    __asm__ (".align 8\n\t"
	     MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    __asm__ (MMX_YUV_MUL : :);
    __asm__ (MMX_YUV_ADD : :);
    __asm__ (MMX_UNPACK_16BGR : : "r" (image));
    
    py += 8;
    pu += 4;
    pv += 4;
    image += 16;
    x += 8;

    __asm__ (".align 8\n\t"
	     MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    __asm__ (MMX_YUV_MUL : :);
    __asm__ (MMX_YUV_ADD : :);
    __asm__ (MMX_UNPACK_16BGR : : "r" (image));
    
    if (even) {
      pu -= 4;
      pv -= 4;
    } else {
      pu += (uv_stride - 4);
      pv += (uv_stride - 4);
    }
    
    py += (y_stride - 8);
    image += (rgb_stride - 16);
    
    even = (!even);
  };
}

static void yuv420_bgr16_mmx_anamorphic (uint8_t *image,
					 uint8_t *py, uint8_t *pu, 
					 uint8_t *pv,
					 int rgb_stride, int y_stride, 
					 int uv_stride)
{
  int even = 1;
  int y = 0;
  
  for (y=0; y<16; y++) {
    
      /*
       * this code deals with SINGLE scan line at a time,
       * it converts 8 pixels in each iteration
	     */
    __asm__ (".align 8\n\t"
	     MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    __asm__ (MMX_YUV_MUL : :);
    __asm__ (MMX_YUV_ADD : :);
    __asm__ (MMX_UNPACK_16BGR_ANAMORPHIC : : "r" (image));
    
    py += 8;
    pu += 4;
    pv += 4;
    image += 20;
    __asm__ (".align 8\n\t"
	     MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    __asm__ (MMX_YUV_MUL : :);
    __asm__ (MMX_YUV_ADD : :);
    __asm__ (MMX_UNPACK_16BGR_ANAMORPHIC : : "r" (image));
    
    if (even) {
      pu -= 4;
      pv -= 4;
    } else {
      pu += (uv_stride - 4);
      pv += (uv_stride - 4);
    }
    
    py += (y_stride - 8);
    image += (rgb_stride - 20);
    
    even = (!even);
  };
}


static void yuv420_argb32_mmx (uint8_t * image, uint8_t * py,
			       uint8_t * pu, uint8_t * pv,
			       int rgb_stride, int y_stride, int uv_stride)
{
  int even = 1;
  int y;

  for (y=0; y<16; y++) {

    /* load data for start of next scan line */

    __asm__ (MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    
    __asm__ (".align 8 \n\t"
	     MMX_YUV_MUL
	     MMX_YUV_ADD
	     MMX_UNPACK_32RGB
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    
    py += 8;
    pu += 4;
    pv += 4;
    image += 32;

    __asm__ (MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    
    __asm__ (".align 8 \n\t"
	     MMX_YUV_MUL
	     MMX_YUV_ADD
	     MMX_UNPACK_32RGB
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    
    if (even) {
      pu -= 4;
      pv -= 4;
    } else {
      pu += (uv_stride - 4);
      pv += (uv_stride - 4);
    }

    py += (y_stride - 8);

    image += (rgb_stride - 32);
    
    even = (!even);
  }
}

static void yuv420_argb32_mmx_anamorphic (uint8_t * image, uint8_t * py,
					  uint8_t * pu, uint8_t * pv,
					  int rgb_stride, int y_stride, 
					  int uv_stride)
{
  int even = 1;
  int y;

  for (y=0; y<16; y++) {

    /* load data for start of next scan line */

    __asm__ (MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    
    __asm__ (".align 8 \n\t"
	     MMX_YUV_MUL
	     MMX_YUV_ADD
	     MMX_UNPACK_32RGB_ANAMORPHIC
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    
    py += 8;
    pu += 4;
    pv += 4;
    image += 40;

    __asm__ (MMX_INIT
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    
    __asm__ (".align 8 \n\t"
	     MMX_YUV_MUL
	     MMX_YUV_ADD
	     MMX_UNPACK_32RGB_ANAMORPHIC
	     : : "r" (py), "r" (pu), "r" (pv), "r" (image));
    
    if (even) {
      pu -= 4;
      pv -= 4;
    } else {
      pu += (uv_stride - 4);
      pv += (uv_stride - 4);
    }

    py += (y_stride - 8);

    image += (rgb_stride - 40);
    
    even = (!even);
  }
}


void yuv2rgb_init_mmx (int bpp, int mode)
{
  if (bpp == 15 || bpp == 16) {
    if (mode == MODE_RGB) {
      yuv2rgb = yuv420_rgb16_mmx;
      yuv2rgb_anamorphic = yuv420_rgb16_mmx_anamorphic;
    }
    if (mode == MODE_BGR) {
      yuv2rgb = yuv420_bgr16_mmx;
      yuv2rgb_anamorphic = yuv420_bgr16_mmx_anamorphic;
    }
  }

  /*
  if (bpp == 24) {
    if (mode == MODE_RGB)
      return NULL;
    else if (mode == MODE_BGR)
      return NULL;
  }
  */
  
  if (bpp == 32) {
    if (mode == MODE_RGB) {
      yuv2rgb = yuv420_argb32_mmx;
      yuv2rgb_anamorphic = yuv420_argb32_mmx_anamorphic;
    }
    /*
    else if (mode == MODE_BGR)
      return NULL;
      */
  }
}

#endif






